@@ -35,34 +35,35 @@ public class X9ECParameters
         ASN1Sequence  seq)
     {
         if (!(seq.getObjectAt(0) instanceof ASN1Integer)
-           || !((ASN1Integer)seq.getObjectAt(0)).getValue().equals(ONE))
+            || !((ASN1Integer)seq.getObjectAt(0)).getValue().equals(ONE))
         {
             throw new IllegalArgumentException("bad version in X9ECParameters");
         }
 
-        X9Curve     x9c = new X9Curve(
-                        X9FieldID.getInstance(seq.getObjectAt(1)),
-                        ASN1Sequence.getInstance(seq.getObjectAt(2)));
+        this.n = ((ASN1Integer)seq.getObjectAt(4)).getValue();
+
+        if (seq.size() == 6)
+        {
+            this.h = ((ASN1Integer)seq.getObjectAt(5)).getValue();
+        }
+
+        X9Curve x9c = new X9Curve(
+            X9FieldID.getInstance(seq.getObjectAt(1)), n, h,
+            ASN1Sequence.getInstance(seq.getObjectAt(2)));
 
         this.curve = x9c.getCurve();
         Object p = seq.getObjectAt(3);
 
         if (p instanceof X9ECPoint)
         {
-            this.g = ((X9ECPoint)p);
+            this.g = (X9ECPoint)p;
         }
         else
         {
             this.g = new X9ECPoint(curve, (ASN1OctetString)p);
         }
 
-        this.n = ((ASN1Integer)seq.getObjectAt(4)).getValue();
         this.seed = x9c.getSeed();
-
-        if (seq.size() == 6)
-        {
-            this.h = ((ASN1Integer)seq.getObjectAt(5)).getValue();
-        }
     }
 
     public static X9ECParameters getInstance(Object obj)
