@@ -11,10 +11,15 @@ import org.bouncycastle.util.Pack;
  * Implementation of the scrypt a password-based key derivation function.
  * <p>
  * Scrypt was created by Colin Percival and is specified in <a
- * href="http://tools.ietf.org/html/draft-josefsson-scrypt-kdf-01">draft-josefsson-scrypt-kd</a>
+ * href="https://tools.ietf.org/html/rfc7914">RFC 7914 - The scrypt Password-Based Key Derivation Function</a>
  */
 public class SCrypt
 {
+    private SCrypt()
+    {
+         // not used.
+    }
+
     /**
      * Generate a key using the scrypt key derivation function.
      *
