@@ -4,7 +4,8 @@ import java.util.Vector;
 
 import org.bouncycastle.util.Arrays;
 
-public class Tables1kGCMExponentiator implements GCMExponentiator
+public class Tables1kGCMExponentiator
+    implements GCMExponentiator
 {
     // A lookup table of the power-of-two powers of 'x'
     // - lookupPowX2[i] = x^(2^i)
@@ -12,8 +13,8 @@ public class Tables1kGCMExponentiator implements GCMExponentiator
 
     public void init(byte[] x)
     {
-        int[] y = GCMUtil.asInts(x);
-        if (lookupPowX2 != null && Arrays.areEqual(y, (int[])lookupPowX2.elementAt(0)))
+        long[] y = GCMUtil.asLongs(x);
+        if (lookupPowX2 != null && Arrays.areEqual(y, (long[])lookupPowX2.elementAt(0)))
         {
             return;
         }
@@ -24,14 +25,14 @@ public class Tables1kGCMExponentiator implements GCMExponentiator
 
     public void exponentiateX(long pow, byte[] output)
     {
-        int[] y = GCMUtil.oneAsInts();
+        long[] y = GCMUtil.oneAsLongs();
         int bit = 0;
         while (pow > 0)
         {
             if ((pow & 1L) != 0)
             {
                 ensureAvailable(bit);
-                GCMUtil.multiply(y, (int[])lookupPowX2.elementAt(bit));
+                GCMUtil.multiply(y, (long[])lookupPowX2.elementAt(bit));
             }
             ++bit;
             pow >>>= 1;
@@ -45,11 +46,11 @@ public class Tables1kGCMExponentiator implements GCMExponentiator
         int count = lookupPowX2.size();
         if (count <= bit)
         {
-            int[] tmp = (int[])lookupPowX2.elementAt(count - 1);
+            long[] tmp = (long[])lookupPowX2.elementAt(count - 1);
             do
             {
                 tmp = Arrays.clone(tmp);
-                GCMUtil.multiply(tmp, tmp);
+                GCMUtil.square(tmp, tmp);
                 lookupPowX2.addElement(tmp);
             }
             while (++count <= bit);
