@@ -64,14 +64,22 @@ public abstract class ECPoint
         this.zs = zs;
     }
 
-    protected boolean satisfiesCofactor()
-    {
-        BigInteger h = curve.getCofactor();
-        return h == null || h.equals(ECConstants.ONE) || !ECAlgorithms.referenceMultiply(this, h).isInfinity();
-    }
-
     protected abstract boolean satisfiesCurveEquation();
 
+    protected boolean satisfiesOrder()
+    {
+        if (ECConstants.ONE.equals(curve.getCofactor()))
+        {
+            return true;
+        }
+
+        BigInteger n = curve.getOrder();
+
+        // TODO Require order to be available for all curves
+
+        return n == null || ECAlgorithms.referenceMultiply(this, n).isInfinity();
+    }
+
     public final ECPoint getDetachedPoint()
     {
         return normalize().detach();
@@ -90,33 +98,6 @@ public abstract class ECPoint
         return null == curve ? ECCurve.COORD_AFFINE : curve.getCoordinateSystem();
     }
 
-    /**
-     * Normalizes this point, and then returns the affine x-coordinate.
-     * 
-     * Note: normalization can be expensive, this method is deprecated in favour
-     * of caller-controlled normalization.
-     * 
-     * @deprecated Use getAffineXCoord(), or normalize() and getXCoord(), instead
-     */
-    public ECFieldElement getX()
-    {
-        return normalize().getXCoord();
-    }
-
-
-    /**
-     * Normalizes this point, and then returns the affine y-coordinate.
-     * 
-     * Note: normalization can be expensive, this method is deprecated in favour
-     * of caller-controlled normalization.
-     * 
-     * @deprecated Use getAffineYCoord(), or normalize() and getYCoord(), instead
-     */
-    public ECFieldElement getY()
-    {
-        return normalize().getYCoord();
-    }
-
     /**
      * Returns the affine x-coordinate after checking that this point is normalized.
      * 
@@ -296,29 +277,59 @@ public abstract class ECPoint
     }
 
     public boolean isValid()
+    {
+        return implIsValid(true);
+    }
+
+    boolean isValidPartial()
+    {
+        return implIsValid(false);
+    }
+
+    boolean implIsValid(final boolean checkOrder)
     {
         if (isInfinity())
         {
             return true;
         }
 
-        // TODO Sanity-check the field elements
-
-        ECCurve curve = getCurve();
-        if (curve != null)
+        ValidityPrecompInfo validity = (ValidityPrecompInfo)getCurve().precompute(this, ValidityPrecompInfo.PRECOMP_NAME, new PreCompCallback()
         {
-            if (!satisfiesCurveEquation())
+            public PreCompInfo precompute(PreCompInfo existing)
             {
-                return false;
-            }
+                ValidityPrecompInfo info = (existing instanceof ValidityPrecompInfo) ? (ValidityPrecompInfo)existing : null;
+                if (info == null)
+                {
+                    info = new ValidityPrecompInfo();
+                }
 
-            if (!satisfiesCofactor())
-            {
-                return false;
+                if (info.hasFailed())
+                {
+                    return info;
+                }
+                if (!info.hasCurveEquationPassed())
+                {
+                    if (!satisfiesCurveEquation())
+                    {
+                        info.reportFailed();
+                        return info;
+                    }
+                    info.reportCurveEquationPassed();
+                }
+                if (checkOrder && !info.hasOrderPassed())
+                {
+                    if (!satisfiesOrder())
+                    {
+                        info.reportFailed();
+                        return info;
+                    }
+                    info.reportOrderPassed();
+                }
+                return info;
             }
-        }
+        });
 
-        return true;
+        return !validity.hasFailed();
     }
 
     public ECPoint scaleX(ECFieldElement scale)
@@ -602,20 +613,6 @@ public abstract class ECPoint
      */
     public static class Fp extends AbstractFp
     {
-        /**
-         * Create a point which encodes without point compression.
-         * 
-         * @param curve the curve to use
-         * @param x affine x co-ordinate
-         * @param y affine y co-ordinate
-         * 
-         * @deprecated Use ECCurve.createPoint to construct points
-         */
-        public Fp(ECCurve curve, ECFieldElement x, ECFieldElement y)
-        {
-            this(curve, x, y, false);
-        }
-
         /**
          * Create a point that encodes with or without point compression.
          * 
@@ -647,7 +644,7 @@ public abstract class ECPoint
 
         protected ECPoint detach()
         {
-            return new ECPoint.Fp(null, this.getAffineXCoord(), this.getAffineYCoord());
+            return new ECPoint.Fp(null, this.getAffineXCoord(), this.getAffineYCoord(), false);
         }
 
         public ECFieldElement getZCoord(int index)
@@ -1424,6 +1421,46 @@ public abstract class ECPoint
             return lhs.equals(rhs);
         }
 
+        protected boolean satisfiesOrder()
+        {
+            BigInteger cofactor = curve.getCofactor();
+            if (ECConstants.TWO.equals(cofactor))
+            {
+                /*
+                 *  Check that the trace of (X + A) is 0, then there exists a solution to L^2 + L = X + A,
+                 *  and so a halving is possible, so this point is the double of another.  
+                 */
+                ECPoint N = this.normalize();
+                ECFieldElement X = N.getAffineXCoord();
+                ECFieldElement rhs = X.add(curve.getA());
+                return ((ECFieldElement.AbstractF2m)rhs).trace() == 0;
+            }
+            if (ECConstants.FOUR.equals(cofactor))
+            {
+                /*
+                 * Solve L^2 + L = X + A to find the half of this point, if it exists (fail if not).
+                 * Generate both possibilities for the square of the half-point's x-coordinate (w),
+                 * and check if Tr(w + A) == 0 for at least one; then a second halving is possible
+                 * (see comments for cofactor 2 above), so this point is four times another.
+                 * 
+                 * Note: Tr(x^2) == Tr(x). 
+                 */
+                ECPoint N = this.normalize();
+                ECFieldElement X = N.getAffineXCoord();
+                ECFieldElement lambda = ((ECCurve.AbstractF2m)curve).solveQuadraticEquation(X.add(curve.getA()));
+                if (lambda == null)
+                {
+                    return false;
+                }
+                ECFieldElement w = X.multiply(lambda).add(N.getAffineYCoord()); 
+                ECFieldElement t = w.add(curve.getA());
+                return ((ECFieldElement.AbstractF2m)t).trace() == 0
+                    || ((ECFieldElement.AbstractF2m)(t.add(X))).trace() == 0;
+            }
+
+            return super.satisfiesOrder();
+        }
+
         public ECPoint scaleX(ECFieldElement scale)
         {
             if (this.isInfinity())
@@ -1577,18 +1614,6 @@ public abstract class ECPoint
      */
     public static class F2m extends AbstractF2m
     {
-        /**
-         * @param curve base curve
-         * @param x x point
-         * @param y y point
-         * 
-         * @deprecated Use ECCurve.createPoint to construct points
-         */
-        public F2m(ECCurve curve, ECFieldElement x, ECFieldElement y)
-        {
-            this(curve, x, y, false);
-        }
-        
         /**
          * @param curve base curve
          * @param x x point
@@ -1634,7 +1659,7 @@ public abstract class ECPoint
 
         protected ECPoint detach()
         {
-            return new ECPoint.F2m(null, this.getAffineXCoord(), this.getAffineYCoord()); // earlier JDK
+            return new ECPoint.F2m(null, this.getAffineXCoord(), this.getAffineYCoord(), false); // earlier JDK
         }
 
         public ECFieldElement getYCoord()
