@@ -3,6 +3,8 @@ package org.bouncycastle.pqc.math.linearalgebra;
 import java.math.BigInteger;
 import java.security.SecureRandom;
 
+import org.bouncycastle.crypto.CryptoServicesRegistrar;
+
 /**
  * Class of number-theory related functions for use with integers represented as
  * <tt>int</tt>'s or <tt>BigInteger</tt> objects.
@@ -1051,7 +1053,7 @@ public final class IntegerFunctions
     {
         if (sr == null)
         {
-            sr = new SecureRandom();
+            sr = CryptoServicesRegistrar.getSecureRandom();
         }
         return randomize(upperBound, sr);
     }
@@ -1064,7 +1066,7 @@ public final class IntegerFunctions
 
         if (prng == null)
         {
-            prng = sr != null ? sr : new SecureRandom();
+            prng = sr != null ? sr : CryptoServicesRegistrar.getSecureRandom();
         }
 
         for (int i = 0; i < 20; i++)
