@@ -1055,6 +1055,22 @@ public final class Arrays
         return result;
     }
 
+    /**
+     * Fill input array by zeros
+     *
+     * @param array input array
+     */
+    public static void clear(byte[] array)
+    {
+        if (array != null)
+        {
+            for (int i = 0; i < array.length; i++)
+            {
+                array[i] = 0;
+            }
+        }
+    }
+
     /**
      * Iterator backed by a specific array.
      */
