@@ -72,7 +72,9 @@ public class RegressionTest
         new GeneralNameTest(),
         new ObjectIdentifierTest(),
         new RFC4519Test(),
-        new PolicyConstraintsTest()
+        new PolicyConstraintsTest(),
+        new PollReqContentTest(),
+        new DhSigStaticTest()
     };
 
     public static void main(
