@@ -15,6 +15,7 @@ import org.bouncycastle.crypto.agreement.ECDHCBasicAgreement;
 import org.bouncycastle.crypto.agreement.ECDHCUnifiedAgreement;
 import org.bouncycastle.crypto.agreement.ECMQVBasicAgreement;
 import org.bouncycastle.crypto.digests.SHA3Digest;
+import org.bouncycastle.crypto.ec.CustomNamedCurves;
 import org.bouncycastle.crypto.generators.ECKeyPairGenerator;
 import org.bouncycastle.crypto.params.ECDHUPrivateParameters;
 import org.bouncycastle.crypto.params.ECDHUPublicParameters;
@@ -127,7 +128,7 @@ public class ECTest
             fail("y uncompressed incorrectly");
         }
 
-        byte[] encoding = p.getEncoded();
+        byte[] encoding = p.getEncoded(true);
 
         if (!areEqual(encoding, Hex.decode("03188da80eb03090f67cbf20eb43a18800f4ff0afd82ff1012")))
         {
@@ -737,7 +738,7 @@ public class ECTest
     /**
      * Basic Key Agreement Test
      */
-    private void testECBasicAgreementTest()
+    private void testECDHBasicAgreement()
     {
         SecureRandom random = new SecureRandom();
 
@@ -799,6 +800,34 @@ public class ECTest
         }
     }
 
+    private void testECDHBasicAgreementCofactor()
+    {
+        SecureRandom random = new SecureRandom();
+
+        X9ECParameters x9 = CustomNamedCurves.getByName("curve25519");
+        ECDomainParameters ec = new ECDomainParameters(x9.getCurve(), x9.getG(), x9.getN(), x9.getH(), x9.getSeed());
+
+        ECKeyPairGenerator kpg = new ECKeyPairGenerator();
+        kpg.init(new ECKeyGenerationParameters(ec, random));
+
+        AsymmetricCipherKeyPair p1 = kpg.generateKeyPair();
+        AsymmetricCipherKeyPair p2 = kpg.generateKeyPair();
+
+        BasicAgreement e1 = new ECDHBasicAgreement();
+        BasicAgreement e2 = new ECDHBasicAgreement();
+
+        e1.init(p1.getPrivate());
+        e2.init(p2.getPrivate());
+
+        BigInteger k1 = e1.calculateAgreement(p2.getPublic());
+        BigInteger k2 = e2.calculateAgreement(p1.getPublic());
+
+        if (!k1.equals(k2))
+        {
+            fail("calculated agreement test failed");
+        }
+    }
+
     private void testECMQVTestVector1()
     {
         // Test Vector from GEC-2
@@ -1078,7 +1107,8 @@ public class ECTest
         testECDSA191bitBinary();
         testECDSA239bitBinary();
         testECDSAKeyGenTest();
-        testECBasicAgreementTest();
+        testECDHBasicAgreement();
+        testECDHBasicAgreementCofactor();
 
         testECDSAP224sha224();
         testECDSAP224OneByteOver();
