@@ -17,6 +17,7 @@ import javax.crypto.SecretKey;
 import javax.crypto.spec.IvParameterSpec;
 import javax.crypto.spec.SecretKeySpec;
 
+import org.bouncycastle.bcpg.HashAlgorithmTags;
 import org.bouncycastle.bcpg.PublicKeyAlgorithmTags;
 import org.bouncycastle.bcpg.SymmetricKeyAlgorithmTags;
 import org.bouncycastle.jcajce.util.JcaJceHelper;
@@ -35,12 +36,63 @@ class OperatorHelper
         this.helper = helper;
     }
 
+    /**
+     * Return an appropriate name for the hash algorithm represented by the passed
+     * in hash algorithm ID number (JCA message digest naming convention).
+     *
+     * @param hashAlgorithm the algorithm ID for a hash algorithm.
+     * @return a String representation of the hash name.
+     */
+    String getDigestName(
+        int hashAlgorithm)
+        throws PGPException
+    {
+        switch (hashAlgorithm)
+        {
+        case HashAlgorithmTags.SHA1:
+            return "SHA-1";
+        case HashAlgorithmTags.MD2:
+            return "MD2";
+        case HashAlgorithmTags.MD5:
+            return "MD5";
+        case HashAlgorithmTags.RIPEMD160:
+            return "RIPEMD160";
+        case HashAlgorithmTags.SHA256:
+            return "SHA-256";
+        case HashAlgorithmTags.SHA384:
+            return "SHA-384";
+        case HashAlgorithmTags.SHA512:
+            return "SHA-512";
+        case HashAlgorithmTags.SHA224:
+            return "SHA-224";
+        case HashAlgorithmTags.TIGER_192:
+            return "TIGER";
+        default:
+            throw new PGPException("unknown hash algorithm tag in getDigestName: " + hashAlgorithm);
+        }
+    }
+
     MessageDigest createDigest(int algorithm)
         throws GeneralSecurityException, PGPException
     {
         MessageDigest dig;
 
-        dig = helper.createDigest(PGPUtil.getDigestName(algorithm));
+        String digestName = getDigestName(algorithm);
+        try
+        {
+            dig = helper.createDigest(digestName);
+        }
+        catch (NoSuchAlgorithmException e)
+        {
+            if (algorithm >= HashAlgorithmTags.SHA256 && algorithm <= HashAlgorithmTags.SHA224)
+            {
+                dig = helper.createDigest("SHA" + digestName.substring(4));
+            }
+            else
+            {
+                throw e;
+            }
+        }
 
         return dig;
     }
