@@ -4,8 +4,6 @@ import java.io.IOException;
 import java.util.Hashtable;
 import java.util.Vector;
 
-import org.bouncycastle.tls.crypto.DHGroup;
-import org.bouncycastle.tls.crypto.DHStandardGroups;
 import org.bouncycastle.tls.crypto.TlsCipher;
 import org.bouncycastle.tls.crypto.TlsCrypto;
 import org.bouncycastle.tls.crypto.TlsCryptoParameters;
@@ -33,8 +31,9 @@ public abstract class AbstractTlsServer
     protected short maxFragmentLengthOffered;
     protected boolean truncatedHMacOffered;
     protected Vector supportedSignatureAlgorithms;
-    protected int[] namedCurves;
+    protected int[] clientSupportedGroups;
     protected short[] clientECPointFormats, serverECPointFormats;
+    protected CertificateStatusRequest certificateStatusRequest;
 
     protected ProtocolVersion serverVersion;
     protected int selectedCipherSuite;
@@ -72,12 +71,7 @@ public abstract class AbstractTlsServer
 
     protected short[] getCompressionMethods()
     {
-        return new short[]{CompressionMethod._null};
-    }
-
-    protected DHGroup getDHParameters()
-    {
-        return DHStandardGroups.rfc3526_2048;
+        return new short[]{ CompressionMethod._null };
     }
 
     protected ProtocolVersion getMaximumVersion()
@@ -94,29 +88,45 @@ public abstract class AbstractTlsServer
     {
         // NOTE: BC supports all the current set of point formats so we don't check them here
 
-        if (namedCurves == null)
+        if (clientSupportedGroups == null)
         {
             /*
              * RFC 4492 4. A client that proposes ECC cipher suites may choose not to include these
              * extensions. In this case, the server is free to choose any one of the elliptic curves
              * or point formats [...].
              */
-            return NamedCurve.getMaximumCurveBits();
+            return NamedGroup.getMaximumCurveBits();
         }
 
         int maxBits = 0;
-        for (int i = 0; i < namedCurves.length; ++i)
+        for (int i = 0; i < clientSupportedGroups.length; ++i)
         {
-            maxBits = Math.max(maxBits, NamedCurve.getCurveBits(namedCurves[i]));
+            maxBits = Math.max(maxBits, NamedGroup.getCurveBits(clientSupportedGroups[i]));
         }
         return maxBits;
     }
 
-    protected boolean isSelectableCipherSuite(int cipherSuite, int availCurveBits, Vector sigAlgs)
+    protected int getMaximumNegotiableFiniteFieldBits()
+    {
+        if (clientSupportedGroups == null)
+        {
+            return NamedGroup.getMaximumFiniteFieldBits();
+        }
+
+        int maxBits = 0;
+        for (int i = 0; i < clientSupportedGroups.length; ++i)
+        {
+            maxBits = Math.max(maxBits, NamedGroup.getFiniteFieldBits(clientSupportedGroups[i]));
+        }
+        return maxBits;
+    }
+
+    protected boolean isSelectableCipherSuite(int cipherSuite, int availCurveBits, int availFiniteFieldBits, Vector sigAlgs)
     {
         return Arrays.contains(this.offeredCipherSuites, cipherSuite)
             && TlsUtils.isValidCipherSuiteForVersion(cipherSuite, serverVersion)
             && availCurveBits >= TlsECCUtils.getMinimumCurveBits(cipherSuite)
+            && availFiniteFieldBits >= TlsDHUtils.getMinimumFiniteFieldBits(cipherSuite)
             && TlsUtils.isValidCipherSuiteForSignatureAlgorithms(cipherSuite, sigAlgs);
     }
 
@@ -128,18 +138,18 @@ public abstract class AbstractTlsServer
 
     protected int selectCurve(int minimumCurveBits)
     {
-        if (namedCurves == null)
+        if (clientSupportedGroups == null)
         {
             return selectDefaultCurve(minimumCurveBits);
         }
 
-        // Try to find a supported named curve of the required size from the client's list.
-        for (int i = 0; i < namedCurves.length; ++i)
+        // Try to find a supported named group of the required size from the client's list.
+        for (int i = 0; i < clientSupportedGroups.length; ++i)
         {
-            int namedCurve = namedCurves[i];
-            if (NamedCurve.getCurveBits(namedCurve) >= minimumCurveBits)
+            int namedGroup = clientSupportedGroups[i];
+            if (NamedGroup.getCurveBits(namedGroup) >= minimumCurveBits)
             {
-                return namedCurve;
+                return namedGroup;
             }
         }
 
@@ -148,32 +158,71 @@ public abstract class AbstractTlsServer
 
     protected int selectDefaultCurve(int minimumCurveBits)
     {
-        // Note: this must all have a co-factor of 1 to qualify for FIPS ECDH.
-        return minimumCurveBits <= 256 ? NamedCurve.secp256r1
-            :  minimumCurveBits <= 384 ? NamedCurve.secp384r1
-            :  minimumCurveBits <= 521 ? NamedCurve.secp521r1
+        return minimumCurveBits <= 256 ? NamedGroup.secp256r1
+            :  minimumCurveBits <= 384 ? NamedGroup.secp384r1
+            :  minimumCurveBits <= 521 ? NamedGroup.secp521r1
+            :  minimumCurveBits <= 571 ? NamedGroup.sect571r1
             :  -1;
     }
 
-    protected TlsDHConfig selectDHConfig()
+    protected TlsDHConfig selectDefaultDHConfig(int minimumFiniteFieldBits)
     {
-        return TlsDHUtils.selectDHConfig(getDHParameters()); 
+        int namedGroup = minimumFiniteFieldBits <= 2048 ? NamedGroup.ffdhe2048
+                      :  minimumFiniteFieldBits <= 3072 ? NamedGroup.ffdhe3072
+                      :  minimumFiniteFieldBits <= 4096 ? NamedGroup.ffdhe4096
+                      :  minimumFiniteFieldBits <= 6144 ? NamedGroup.ffdhe6144
+                      :  minimumFiniteFieldBits <= 8192 ? NamedGroup.ffdhe8192
+                      :  -1;
+
+        return TlsDHUtils.createNamedDHConfig(namedGroup);
+    }
+
+    protected TlsDHConfig selectDHConfig() throws IOException
+    {
+        int minimumFiniteFieldBits = TlsDHUtils.getMinimumFiniteFieldBits(selectedCipherSuite);
+
+        TlsDHConfig dhConfig = selectDHConfig(minimumFiniteFieldBits);
+        if (dhConfig == null)
+        {
+            throw new TlsFatalAlert(AlertDescription.internal_error);
+        }
+        return dhConfig;
+    }
+
+    protected TlsDHConfig selectDHConfig(int minimumFiniteFieldBits)
+    {
+        if (clientSupportedGroups == null)
+        {
+            return selectDefaultDHConfig(minimumFiniteFieldBits);
+        }
+
+        // Try to find a supported named group of the required size from the client's list.
+        for (int i = 0; i < clientSupportedGroups.length; ++i)
+        {
+            int namedGroup = clientSupportedGroups[i];
+            if (NamedGroup.getFiniteFieldBits(namedGroup) >= minimumFiniteFieldBits)
+            {
+                return new TlsDHConfig(namedGroup);
+            }
+        }
+
+        return null;
     }
 
     protected TlsECConfig selectECConfig() throws IOException
     {
         int minimumCurveBits = TlsECCUtils.getMinimumCurveBits(selectedCipherSuite);
 
-        int namedCurve = selectCurve(minimumCurveBits);
-        if (namedCurve < 0)
+        int namedGroup = selectCurve(minimumCurveBits);
+        if (namedGroup < 0)
         {
             throw new TlsFatalAlert(AlertDescription.internal_error);
         }
 
-        boolean compressed = TlsECCUtils.isCompressionPreferred(clientECPointFormats, namedCurve);
+        boolean compressed = TlsECCUtils.isCompressionPreferred(clientECPointFormats, namedGroup);
 
         TlsECConfig ecConfig = new TlsECConfig();
-        ecConfig.setNamedCurve(namedCurve);
+        ecConfig.setNamedGroup(namedGroup);
         ecConfig.setPointCompression(compressed);
         return ecConfig;
     }
@@ -183,6 +232,11 @@ public abstract class AbstractTlsServer
         this.context = context;
     }
 
+    public TlsSession getSessionToResume(byte[] sessionID)
+    {
+        return null;
+    }
+
     public void notifyClientVersion(ProtocolVersion clientVersion)
         throws IOException
     {
@@ -245,8 +299,10 @@ public abstract class AbstractTlsServer
                 }
             }
 
-            this.namedCurves = TlsECCUtils.getSupportedEllipticCurvesExtension(clientExtensions);
+            this.clientSupportedGroups = TlsExtensionsUtils.getSupportedGroupsExtension(clientExtensions);
             this.clientECPointFormats = TlsECCUtils.getSupportedPointFormatsExtension(clientExtensions);
+
+            this.certificateStatusRequest = TlsExtensionsUtils.getStatusRequestExtension(clientExtensions);
         }
 
         /*
@@ -254,10 +310,12 @@ public abstract class AbstractTlsServer
          * does not propose any ECC cipher suites.
          * 
          * NOTE: This was overly strict as there may be ECC cipher suites that we don't recognize.
-         * Also, draft-ietf-tls-negotiated-ff-dhe will be overloading the 'elliptic_curves'
-         * extension to explicitly allow FFDHE (i.e. non-ECC) groups.
+         * Also, RFC 7919 renamed 'elliptic_curves' to 'supported_groups' and now allows FFDHE (i.e.
+         * non-ECC) groups. If ec_point_formats are unnecessarily present, it doesn't seem worth
+         * failing the handshake over.
          */
-//        if (!this.eccCipherSuitesOffered && (this.namedCurves != null || this.clientECPointFormats != null))
+//        if ((this.clientSupportedGroups != null || this.clientECPointFormats != null)
+//            && !TlsECCUtils.containsECCipherSuites(offeredCipherSuites))
 //        {
 //            throw new TlsFatalAlert(AlertDescription.illegal_parameter);
 //        }
@@ -300,12 +358,13 @@ public abstract class AbstractTlsServer
          * formats supported by the client [...].
          */
         int availCurveBits = getMaximumNegotiableCurveBits();
+        int availFiniteFieldBits = getMaximumNegotiableFiniteFieldBits();
 
         int[] cipherSuites = getCipherSuites();
         for (int i = 0; i < cipherSuites.length; ++i)
         {
             int cipherSuite = cipherSuites[i];
-            if (isSelectableCipherSuite(cipherSuite, availCurveBits, sigAlgs)
+            if (isSelectableCipherSuite(cipherSuite, availCurveBits, availFiniteFieldBits, sigAlgs)
                 && selectCipherSuite(cipherSuite))
             {
                 return cipherSuite;
@@ -369,6 +428,16 @@ public abstract class AbstractTlsServer
             TlsECCUtils.addSupportedPointFormatsExtension(checkServerExtensions(), serverECPointFormats);
         }
 
+        if (this.certificateStatusRequest != null)
+        {
+            /*
+             * RFC 6066 8. If a server returns a "CertificateStatus" message, then the server MUST
+             * have included an extension of type "status_request" with empty "extension_data" in
+             * the extended server hello.
+             */
+            checkServerExtensions().put(TlsExtensionsUtils.EXT_status_request, TlsExtensionsUtils.createEmptyExtensionData());
+        }
+
         return serverExtensions;
     }
 
