@@ -2,6 +2,9 @@ package org.bouncycastle.tls;
 
 import org.bouncycastle.tls.crypto.TlsSecret;
 
+/**
+ * Carrier class for general security parameters.
+ */
 public class SecurityParameters
 {
     int entity = -1;
@@ -16,9 +19,11 @@ public class SecurityParameters
     byte[] sessionHash = null;
     byte[] pskIdentity = null;
     byte[] srpIdentity = null;
+    byte[] tlsServerEndPoint = null;
     byte[] tlsUnique = null;
     boolean encryptThenMAC = false;
     boolean extendedMasterSecret = false;
+    boolean extendedPadding = false;
     boolean truncatedHMac = false;
 
     void clear()
@@ -113,6 +118,11 @@ public class SecurityParameters
         return srpIdentity;
     }
 
+    public byte[] getTLSServerEndPoint()
+    {
+        return tlsServerEndPoint;
+    }
+
     public byte[] getTLSUnique()
     {
         return tlsUnique;
@@ -128,6 +138,11 @@ public class SecurityParameters
         return extendedMasterSecret;
     }
 
+    public boolean isExtendedPadding()
+    {
+        return extendedPadding;
+    }
+
     public boolean isTruncatedHMac()
     {
         return truncatedHMac;
