@@ -5,6 +5,7 @@ import java.io.ByteArrayOutputStream;
 import java.io.EOFException;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.InterruptedIOException;
 import java.io.OutputStream;
 import java.util.Enumeration;
 import java.util.Hashtable;
@@ -67,7 +68,9 @@ public abstract class TlsProtocol
     private volatile boolean failedWithError = false;
     private volatile boolean appDataReady = false;
     private volatile boolean appDataSplitEnabled = true;
+    private volatile boolean resumableHandshake = false;
     private volatile int appDataSplitMode = ADS_MODE_1_Nsub1;
+
     // TODO[tls-ops] Investigate whether we can handle (expected/actual) verify data using TlsSecret
     private byte[] expected_verify_data = null;
 
@@ -107,6 +110,25 @@ public abstract class TlsProtocol
         this.recordStream = new RecordStream(this, input, output);
     }
 
+    public void resumeHandshake() throws IOException
+    {
+        if (!blocking)
+        {
+            throw new IllegalStateException("Cannot use resumeHandshake() in non-blocking mode!");
+        }
+        if (!isHandshaking())
+        {
+            throw new IllegalStateException("No handshake in progress");
+        }
+
+        blockForHandshake();
+    }
+
+    protected void closeConnection() throws IOException
+    {
+        recordStream.close();
+    }
+
     protected abstract TlsContext getContext();
 
     abstract AbstractTlsContext getContextAdmin();
@@ -165,27 +187,32 @@ public abstract class TlsProtocol
 
             raiseAlertWarning(AlertDescription.close_notify, "Connection closed");
 
-            recordStream.safeClose();
-
             if (!appDataReady)
             {
                 cleanupHandshake();
             }
+
+            closeConnection();
         }
     }
 
-    protected void handleException(short alertDescription, String message, Throwable cause)
+    protected void handleException(short alertDescription, String message, Throwable e)
         throws IOException
     {
+        if ((appDataReady || isResumableHandshake()) && (e instanceof InterruptedIOException))
+        {
+            return;
+        }
+
         if (!closed)
         {
-            raiseAlertFatal(alertDescription, message, cause);
+            raiseAlertFatal(alertDescription, message, e);
 
             handleFailure();
         }
     }
 
-    protected void handleFailure()
+    protected void handleFailure() throws IOException
     {
         this.closed = true;
         this.failedWithError = true;
@@ -197,12 +224,12 @@ public abstract class TlsProtocol
         // TODO This isn't quite in the right place. Also, as of TLS 1.1 the above is obsolete.
         invalidateSession();
 
-        recordStream.safeClose();
-
         if (!appDataReady)
         {
             cleanupHandshake();
         }
+
+        closeConnection();
     }
 
     protected abstract void handleHandshakeMessage(short type, ByteArrayInputStream buf)
@@ -233,6 +260,20 @@ public abstract class TlsProtocol
         }
     }
 
+    protected void blockForHandshake() throws IOException
+    {
+        while (this.connection_state != CS_END)
+        {
+            if (isClosed())
+            {
+                // NOTE: Any close during the handshake should have raised an exception.
+                throw new TlsFatalAlert(AlertDescription.internal_error);
+            }
+
+            safeReadRecord();
+        }
+    }
+
     protected void cleanupHandshake()
     {
         if (this.expected_verify_data != null)
@@ -258,23 +299,6 @@ public abstract class TlsProtocol
         this.allowCertificateStatus = false;
         this.expectSessionTicket = false;
     }
-    
-    protected void blockForHandshake() throws IOException
-    {
-        if (blocking)
-        {
-            while (this.connection_state != CS_END)
-            {
-                if (this.closed)
-                {
-                    // NOTE: Any close during the handshake should have raised an exception.
-                    throw new TlsFatalAlert(AlertDescription.internal_error);
-                }
-
-                safeReadRecord();
-            }
-        }
-    }
 
     protected void completeHandshake()
         throws IOException
@@ -553,6 +577,10 @@ public abstract class TlsProtocol
                 throw new IllegalStateException("Cannot read application data until initial handshake completed.");
             }
 
+            /*
+             * NOTE: Only called more than once when empty records are received, so no special
+             * InterruptedIOException handling is necessary.
+             */
             safeReadRecord();
         }
 
@@ -626,12 +654,12 @@ public abstract class TlsProtocol
         throw new TlsNoCloseNotifyException();
     }
 
-    protected boolean safeReadFullRecord(byte[] record)
+    protected boolean safeReadFullRecord(byte[] input, int inputOff, int inputLen)
         throws IOException
     {
         try
         {
-            return recordStream.readFullRecord(record);
+            return recordStream.readFullRecord(input, inputOff, inputLen);
         }
         catch (TlsFatalAlert e)
         {
@@ -724,7 +752,8 @@ public abstract class TlsProtocol
                  * 
                  * DO NOT REMOVE THIS CODE, EXCEPT YOU KNOW EXACTLY WHAT YOU ARE DOING HERE.
                  */
-                switch (appDataSplitMode) {
+                switch (getAppDataSplitMode())
+                {
                     case ADS_MODE_0_N_FIRSTONLY:
                         this.appDataSplitEnabled = false;
                         // fall through intended!
@@ -751,7 +780,13 @@ public abstract class TlsProtocol
         }
     }
 
-    protected void setAppDataSplitMode(int appDataSplitMode) {
+    public int getAppDataSplitMode()
+    {
+        return appDataSplitMode;
+    }
+
+    public void setAppDataSplitMode(int appDataSplitMode)
+    {
         if (appDataSplitMode < ADS_MODE_1_Nsub1 ||
             appDataSplitMode > ADS_MODE_0_N_FIRSTONLY)
         {
@@ -760,6 +795,16 @@ public abstract class TlsProtocol
         this.appDataSplitMode = appDataSplitMode;
 	}
 
+    public boolean isResumableHandshake()
+    {
+        return resumableHandshake;
+    }
+
+    public void setResumableHandshake(boolean resumableHandshake)
+    {
+        this.resumableHandshake = resumableHandshake;
+    }
+
     protected void writeHandshakeMessage(byte[] buf, int off, int len) throws IOException
     {
         if (len < 4)
@@ -878,7 +923,7 @@ public abstract class TlsProtocol
 
         if (this.appDataSplitEnabled)
         {
-            switch (appDataSplitMode)
+            switch (getAppDataSplitMode())
             {
                 case ADS_MODE_0_N_FIRSTONLY:
                 case ADS_MODE_0_N:
@@ -906,6 +951,17 @@ public abstract class TlsProtocol
         }
     }
 
+    /**
+     * Equivalent to <code>offerInput(input, 0, input.length)</code>
+     * @see TlsProtocol#offerInput(byte[], int, int)
+     * @param input The input buffer to offer
+     * @throws IOException If an error occurs while decrypting or processing a record
+     */
+    public void offerInput(byte[] input) throws IOException
+    {
+        offerInput(input, 0, input.length);
+    }
+
     /**
      * Offer input from an arbitrary source. Only allowed in non-blocking mode.<br>
      * <br>
@@ -922,22 +978,23 @@ public abstract class TlsProtocol
      * You should always check to see if there is any available output after calling
      * this method by calling {@link #getAvailableOutputBytes()}.
      * @param input The input buffer to offer
+     * @param inputOff The offset within the input buffer that input begins
+     * @param inputLen The number of bytes of input being offered
      * @throws IOException If an error occurs while decrypting or processing a record
      */
-    public void offerInput(byte[] input) throws IOException
+    public void offerInput(byte[] input, int inputOff, int inputLen) throws IOException
     {
         if (blocking)
         {
             throw new IllegalStateException("Cannot use offerInput() in blocking mode! Use getInputStream() instead.");
         }
-        
         if (closed)
         {
             throw new IOException("Connection is closed, cannot accept any more input");
         }
 
         // Fast path if the input is arriving one record at a time
-        if (inputBuffers.available() == 0 && safeReadFullRecord(input))
+        if (inputBuffers.available() == 0 && safeReadFullRecord(input, inputOff, inputLen))
         {
             if (closed)
             {
@@ -950,13 +1007,16 @@ public abstract class TlsProtocol
             return;
         }
 
-        inputBuffers.addBytes(input);
+        inputBuffers.addBytes(input, inputOff, inputLen);
 
         // loop while there are enough bytes to read the length of the next record
         while (inputBuffers.available() >= RecordFormat.FRAGMENT_OFFSET)
         {
             byte[] recordHeader = new byte[RecordFormat.FRAGMENT_OFFSET];
-            inputBuffers.peek(recordHeader);
+            if (RecordFormat.FRAGMENT_OFFSET != inputBuffers.peek(recordHeader))
+            {
+                throw new TlsFatalAlert(AlertDescription.internal_error);
+            }
 
             RecordPreview preview = safePreviewRecordHeader(recordHeader);
             if (inputBuffers.available() < preview.getRecordSize())
@@ -965,6 +1025,7 @@ public abstract class TlsProtocol
                 break;
             }
 
+            // NOTE: This is actually reading from inputBuffers, so InterruptedIOException shouldn't be possible
             safeReadRecord();
 
             if (closed)
@@ -1144,8 +1205,15 @@ public abstract class TlsProtocol
         safeWriteRecord(ContentType.alert, alert, 0, 2);
     }
 
+    /** @deprecated */
     protected void sendCertificateMessage(Certificate certificate)
         throws IOException
+    {
+        sendCertificateMessage(certificate, null);
+    }
+
+    protected void sendCertificateMessage(Certificate certificate, OutputStream endPointHash)
+        throws IOException
     {
         if (certificate == null)
         {
@@ -1154,7 +1222,7 @@ public abstract class TlsProtocol
 
         HandshakeMessage message = new HandshakeMessage(HandshakeType.certificate);
 
-        certificate.encode(message);
+        certificate.encode(getContext(), message, endPointHash);
 
         message.writeToRecordStream();
 
@@ -1223,6 +1291,11 @@ public abstract class TlsProtocol
         return closed;
     }
 
+    public boolean isHandshaking()
+    {
+        return securityParameters != null && this.connection_state != CS_END && !isClosed();
+    }
+
     protected short processMaxFragmentLengthExtension(Hashtable clientExtensions, Hashtable serverExtensions,
         short alertDescription)
         throws IOException
