@@ -1,6 +1,7 @@
 package org.bouncycastle.tls;
 
 import java.io.ByteArrayInputStream;
+import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
@@ -75,13 +76,17 @@ public class TlsServerProtocol
         this.tlsServerContext = new TlsServerContextImpl(tlsServer.getCrypto(), securityParameters);
 
         this.securityParameters.serverRandom = createRandomBlock(tlsServer.shouldUseGMTUnixTime(), tlsServerContext);
+        this.securityParameters.extendedPadding = tlsServer.shouldUseExtendedPadding();
 
         this.tlsServer.init(tlsServerContext);
         this.recordStream.init(tlsServerContext);
 
         this.recordStream.setRestrictReadVersion(false);
 
-        blockForHandshake();
+        if (blocking)
+        {
+            blockForHandshake();
+        }
     }
 
     protected void cleanupHandshake()
@@ -150,6 +155,7 @@ public class TlsServerProtocol
 
                 Certificate serverCertificate = null;
 
+                ByteArrayOutputStream endPointHash = new ByteArrayOutputStream();
                 if (this.serverCredentials == null)
                 {
                     this.keyExchange.skipServerCredentials();
@@ -159,8 +165,9 @@ public class TlsServerProtocol
                     this.keyExchange.processServerCredentials(this.serverCredentials);
 
                     serverCertificate = this.serverCredentials.getCertificate();
-                    sendCertificateMessage(serverCertificate);
+                    sendCertificateMessage(serverCertificate, endPointHash);
                 }
+                securityParameters.tlsServerEndPoint = endPointHash.toByteArray();
                 this.connection_state = CS_SERVER_CERTIFICATE;
 
                 // TODO[RFC 3546] Check whether empty certificates is possible, allowed, or excludes CertificateStatus
@@ -441,7 +448,7 @@ public class TlsServerProtocol
     protected void receiveCertificateMessage(ByteArrayInputStream buf)
         throws IOException
     {
-        Certificate clientCertificate = Certificate.parse(getContext(), buf);
+        Certificate clientCertificate = Certificate.parse(getContext(), buf, null);
 
         assertEmpty(buf);
 
