@@ -1,14 +1,18 @@
 package org.bouncycastle.tls.test;
 
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.InterruptedIOException;
 import java.io.OutputStream;
 import java.io.PipedInputStream;
 import java.io.PipedOutputStream;
 import java.security.SecureRandom;
 
-import junit.framework.TestCase;
 import org.bouncycastle.tls.ProtocolVersion;
+import org.bouncycastle.tls.TlsProtocol;
 import org.bouncycastle.util.Arrays;
-import org.bouncycastle.util.io.Streams;
+
+import junit.framework.TestCase;
 
 public class TlsTestCase extends TestCase
 {
@@ -66,8 +70,17 @@ public class TlsTestCase extends TestCase
         NetworkOutputStream clientNetOut = new NetworkOutputStream(clientWrite);
         NetworkOutputStream serverNetOut = new NetworkOutputStream(serverWrite);
 
-        TlsTestClientProtocol clientProtocol = new TlsTestClientProtocol(clientNetIn, clientNetOut, config);
-        TlsTestServerProtocol serverProtocol = new TlsTestServerProtocol(serverNetIn, serverNetOut, config);
+        InterruptedInputStream clientIn = new InterruptedInputStream(clientNetIn, secureRandom);
+        InterruptedInputStream serverIn = new InterruptedInputStream(serverNetIn, secureRandom);
+
+        clientIn.setPercentInterrupted(50);
+        serverIn.setPercentInterrupted(50);
+
+        TlsTestClientProtocol clientProtocol = new TlsTestClientProtocol(clientIn, clientNetOut, config);
+        TlsTestServerProtocol serverProtocol = new TlsTestServerProtocol(serverIn, serverNetOut, config);
+
+        clientProtocol.setResumableHandshake(true);
+        serverProtocol.setResumableHandshake(true);
 
         TlsTestClientImpl clientImpl = new TlsTestClientImpl(config);
         TlsTestServerImpl serverImpl = new TlsTestServerImpl(config);
@@ -78,7 +91,14 @@ public class TlsTestCase extends TestCase
         Exception caught = null;
         try
         {
-            clientProtocol.connect(clientImpl);
+            try
+            {
+                clientProtocol.connect(clientImpl);
+            }
+            catch (InterruptedIOException e)
+            {
+                completeHandshake(clientProtocol);
+            }
 
             // NOTE: Because we write-all before we read-any, this length can't be more than the pipe capacity
             int length = 1000;
@@ -90,11 +110,13 @@ public class TlsTestCase extends TestCase
             output.write(data);
     
             byte[] echo = new byte[data.length];
-            int count = Streams.readFully(clientProtocol.getInputStream(), echo);
+            int count = readFully(clientProtocol.getInputStream(), echo, 0, echo.length);
     
             assertEquals(count, data.length);
             assertTrue(Arrays.areEqual(data, echo));
 
+            assertTrue(Arrays.areEqual(clientImpl.tlsServerEndPoint, serverImpl.tlsServerEndPoint));
+
             assertNotNull(clientImpl.tlsUnique);
             assertNotNull(serverImpl.tlsUnique);
             assertTrue(Arrays.areEqual(clientImpl.tlsUnique, serverImpl.tlsUnique));
@@ -136,6 +158,63 @@ public class TlsTestCase extends TestCase
         }
     }
 
+    static void completeHandshake(TlsProtocol protocol)
+        throws IOException
+    {
+        while (protocol.isHandshaking())
+        {
+            try
+            {
+                protocol.resumeHandshake();
+            }
+            catch (InterruptedIOException e)
+            {
+            }
+        }
+    }
+
+    static int interruptibleRead(InputStream inStr, byte[] buf, int off, int len)
+        throws IOException
+    {
+        for (;;)
+        {
+            try
+            {
+                return inStr.read(buf, off, len);
+            }
+            catch (InterruptedIOException e)
+            {
+            }
+        }
+    }
+
+    static void pipeAll(InputStream inStr, OutputStream outStr)
+        throws IOException
+    {
+        byte[] bs = new byte[4096];
+        int numRead;
+        while ((numRead = interruptibleRead(inStr, bs, 0, bs.length)) >= 0)
+        {
+            outStr.write(bs, 0, numRead);
+        }
+    }
+
+    static int readFully(InputStream inStr, byte[] buf, int off, int len)
+        throws IOException
+    {
+        int totalRead = 0;
+        while (totalRead < len)
+        {
+            int numRead = interruptibleRead(inStr, buf, off + totalRead, len - totalRead);
+            if (numRead < 0)
+            {
+                break;
+            }
+            totalRead += numRead;
+        }
+        return totalRead;
+    }
+
     class ServerThread extends Thread
     {
         protected final TlsTestServerProtocol serverProtocol;
@@ -160,8 +239,16 @@ public class TlsTestCase extends TestCase
         {
             try
             {
-                serverProtocol.accept(serverImpl);
-                Streams.pipeAll(serverProtocol.getInputStream(), serverProtocol.getOutputStream());
+                try
+                {
+                    serverProtocol.accept(serverImpl);
+                }
+                catch (InterruptedIOException e)
+                {
+                    completeHandshake(serverProtocol);
+                }
+
+                pipeAll(serverProtocol.getInputStream(), serverProtocol.getOutputStream());
                 serverProtocol.close();
             }
             catch (Exception e)
